@@ -76,6 +76,8 @@ namespace System.Windows.Forms
 			SetStyle(ControlStyles.ResizeRedraw, true);
 			textEntry.BorderStyle = BorderStyle.None;
 			textEntry.TabStop = false;
+			textEntry.LostFocus += new EventHandler(textentry_LostFocus);
+			textEntry.GotFocus += new EventHandler(textentry_GotFocus);
 			this.Controls.Add(textEntry);
 		
 			popup = new PopupControl();
@@ -591,11 +593,6 @@ namespace System.Windows.Forms
 			base.OnEnter (e);
 			if (DropDownStyle == ComboBoxStyle.DropDownList)
 				Invalidate();
-			else
-			{
-				textEntry.Focus();
-				textEntry.SelectAll();
-			}
 		}
 
 		protected override void OnLeave(EventArgs e)
@@ -952,7 +949,17 @@ namespace System.Windows.Forms
 				}
 			}
 		}
-		
+
+		private void textentry_LostFocus(object sender, EventArgs e)
+		{
+			OnLostFocus(e);
+		}
+
+		private void textentry_GotFocus(object sender, EventArgs e)
+		{
+			OnGotFocus(e);
+		}
+	
 		private void scrollbar_ValueChanged(object sender, EventArgs e)
 		{
 			using (Graphics g = popup.CreateGraphics())
