@@ -59,6 +59,7 @@ public class TextBox : TextBoxBase
 	// Position to draw caret
 	private Rectangle caretBounds;
 	private Pen caretPen;
+	private int caretPosition = 0;
 
 	// Maximum possible X/Y for a region
 	private const int maxXY = 4194304;
@@ -74,6 +75,13 @@ public class TextBox : TextBoxBase
 	// Binding PropertyInfo for DataBinding of Text.
 	private PropertyInfo bindInfo = null;
 	
+	// Variables to work around scrollbar visibility problem
+	private bool showVScrollBar;
+	private bool showHScrollBar;
+
+	private int graphicsOffset;
+
+	[TODO]
 	public TextBox()
 	{
 		// Trap interesting events.  We do it this way rather
@@ -97,7 +105,7 @@ public class TextBox : TextBoxBase
 		BackColor = SystemColors.Window;
 		ForeColor = SystemColors.WindowText;
 
-		// TODO Get this value from SystemInformation
+		// Fix: Get this value from SystemInformation
 		CaretSetPosition(0);
 
 		// Cache the Pen - check what color this should be
@@ -107,6 +115,10 @@ public class TextBox : TextBoxBase
 		SetStyle(ControlStyles.Opaque, true);
 		// Switch on double buffering.
 		SetStyle(ControlStyles.DoubleBuffer | ControlStyles.AllPaintingInWmPaint | ControlStyles.UserPaint, true);
+	
+		// we need this or no text is displayed after first start
+		// this.CreateHandle(); // do we really need this ?????
+		
 	}
 
 	// Gets or sets a value indicating whether pressing ENTER in a multiline TextBox control creates a new line of text in the control or activates the default button for the form.
@@ -122,7 +134,8 @@ public class TextBox : TextBoxBase
 		}
 	}
 
-	// TODO Handle this
+	[TODO]
+	// Handle this
 	// Gets or sets whether the TextBox control modifies the case of characters as they are typed.
 	public CharacterCasing CharacterCasing
 	{
@@ -136,7 +149,8 @@ public class TextBox : TextBoxBase
 		}
 	}
 
-	// TODO Handle this
+	[TODO]
+	// Handle this
 	protected override ImeMode DefaultImeMode
 	{
 		get
@@ -164,7 +178,8 @@ public class TextBox : TextBoxBase
 		}
 	}
 
-	// TODO Handle this
+	[TODO]
+	// Handle this
 	public ScrollBars ScrollBars
 	{
 		get
@@ -182,8 +197,12 @@ public class TextBox : TextBoxBase
 		}
 	}
 
-	private void SetupScrollBars()
+	// Set up the scrollbars. Returns whether SetScrollBarPositions has to be called as well.
+	private bool SetupScrollBars()
 	{
+		// Do we need to call SetScrollBarPositions ?
+		bool callPositions = false;
+		
 		// Set vertical scrollbar
 		// There is no vertical scrollbar if the textbox is not multiline
 		if (Multiline && (scrollBars == ScrollBars.Both || scrollBars == ScrollBars.Vertical))
@@ -194,7 +213,7 @@ public class TextBox : TextBoxBase
 				vScrollBar.backColor = SystemColors.ScrollBar;
 				vScrollBar.ValueChanged+=new EventHandler(vScrollBar_ValueChanged);
 				vScrollBar.Maximum = 0;
-				SetScrollBarPositions();
+				callPositions = true;
 				Controls.Add(vScrollBar);
 			}
 		}
@@ -213,7 +232,7 @@ public class TextBox : TextBoxBase
 				hScrollBar.backColor = SystemColors.ScrollBar;
 				hScrollBar.ValueChanged+=new EventHandler(hScrollBar_ValueChanged);
 				hScrollBar.Maximum = 0;
-				SetScrollBarPositions();
+				callPositions = true;
 				Controls.Add(hScrollBar);
 			}
 		}
@@ -222,6 +241,10 @@ public class TextBox : TextBoxBase
 			Controls.Remove(hScrollBar);
 			hScrollBar = null;
 		}
+
+		SetScrollBarVisibility ();
+		
+		return callPositions;
 	}
 
 	// Setup the positions of the scrollBar depending on the combination
@@ -231,49 +254,22 @@ public class TextBox : TextBoxBase
 		{
 			return;
 		}
-		bool vScrollBarVisible = false;
-		bool hScrollBarVisible = false;
 		
-		int width;
-		if (vScrollBar == null)
-		{
-			width = ClientRectangle.Width;
-		}
-		else
-		{
-			width = ClientRectangle.Width - vScrollBar.Width;
-			if (width < 5)
+		int width = ClientRectangle.Width;
+		if (vScrollBar != null && showVScrollBar)
 			{
-				width = ClientRectangle.Width;
-			}
-			else
-			{
-				vScrollBarVisible = true;
-			}
+			width -= vScrollBar.Width;
 		}
 
-		int height;
-		if (hScrollBar == null)
-		{
-			height = ClientRectangle.Height;
-		}
-		else
-		{
-			height = ClientRectangle.Height - hScrollBar.Height;
-			if (height < 5)
+		int height = ClientRectangle.Height;
+		if (hScrollBar != null && showHScrollBar)
 			{
-				height = ClientRectangle.Height;
-			}
-			else
-			{
-				hScrollBarVisible = true;
-			}
+			height -= hScrollBar.Height;
 		}
 
 
 		if (vScrollBar != null)
 		{
-			vScrollBar.Visible = vScrollBarVisible;
 			vScrollBar.Bounds = new Rectangle(ClientRectangle.Width - vScrollBar.Width, 0, vScrollBar.Width, height);
 			
 			int remainder = TextDrawArea.Height % Font.Height;
@@ -290,7 +286,6 @@ public class TextBox : TextBoxBase
 		}
 		if (hScrollBar != null)
 		{
-			hScrollBar.Visible = hScrollBarVisible;
 			hScrollBar.Bounds = new Rectangle(0, ClientRectangle.Height - hScrollBar.Height, width, hScrollBar.Height);
 			int maximum = MaxTextDimensions.Width;
 			if (maximum < TextDrawArea.Width)
@@ -304,11 +299,33 @@ public class TextBox : TextBoxBase
 		}
 	}
 
+	// Set whether the scrollbars are visible.
+	private void SetScrollBarVisibility()
+	{
+		if (vScrollBar == null && hScrollBar == null)
+		{
+			return;
+		}
+		
+		if (vScrollBar != null)
+		{
+			showVScrollBar = (ClientRectangle.Width - vScrollBar.Width >= 5);
+			vScrollBar.Visible = showVScrollBar;
+		}
+
+		if (hScrollBar != null)
+		{
+			showHScrollBar = (ClientRectangle.Height - hScrollBar.Height >= 5);
+			hScrollBar.Visible = showHScrollBar;
+		}
+	}
+
+	[TODO]
 	private Size MaxTextDimensions
 	{
 		get
 		{
-			// TODO handle case right to left
+			// Fix: handle case right to left
 			int x = 0;
 			int y = 0;
 			if (Text.Length > 0)
@@ -358,37 +375,49 @@ public class TextBox : TextBoxBase
 		}
 		set
 		{
-			// Change all text endings of CR or LF into CRLF
-			System.Text.StringBuilder sb = new System.Text.StringBuilder
-				(value != null ? value.Length : 0);
-			if (value != null && value.Length > 0)
-			{
-				char cPrevious = (char)0;
-				for (int i = 0; i < value.Length; i++)
-				{
-					char c = value[i];
-					bool isNext = (i < value.Length - 1);
-					char cNext = (char)0;
-					if (isNext)
-					{
-						cNext = value[i+1];
-					}
-					if ((!isNext || cNext != '\n') && c == '\r')
-					{
-						sb.Append("\r\n");
-					}
-					else if (c=='\n' && cPrevious != '\r')
-					{
-						sb.Append("\r\n");
-					}
-					else
-					{
-						sb.Append(c);
-					}
-					cPrevious = c;
-				}
+			if( base.Text == value ) {	// check if text has changed
+				return;
 			}
-			SetTextActual( sb.ToString());
+			
+			if( value.IndexOfAny( new char [] { '\n', '\r' } ) >= 0 ) {	// check if chars are in use, before use of StringBuilder (performance)
+			
+				// Change all text endings of CR or LF into CRLF
+				System.Text.StringBuilder sb = new System.Text.StringBuilder
+					(value != null ? value.Length : 0);
+				if (value != null && value.Length > 0)
+				{
+					char cPrevious = (char)0;
+					for (int i = 0; i < value.Length; i++)
+					{
+						char c = value[i];
+						bool isNext = (i < value.Length - 1);
+						char cNext = (char)0;
+						if (isNext)
+						{
+							cNext = value[i+1];
+						}
+						if ((!isNext || cNext != '\n') && c == '\r')
+						{
+							sb.Append("\r\n");
+						}
+						else if (c=='\n' && cPrevious != '\r')
+						{
+							sb.Append("\r\n");
+						}
+						else
+						{
+							sb.Append(c);
+						}
+						cPrevious = c;
+					}
+				}
+				
+				SetTextActual( sb.ToString());
+			}
+			else {
+				SetTextActual( value );
+			}
+			
 			// Set the position to the end
 			SelectInternal(Text.Length, 0);
 			if (!inTextChangedEvent)
@@ -531,6 +560,9 @@ public class TextBox : TextBoxBase
 	// Process when the control receives the focus
 	protected override void OnEnter(EventArgs e)
 	{
+		// reset the caret position
+		CaretSetPosition(caretPosition);
+		
 		// Perform the regular focus handling.
 		base.OnEnter(e);
 		CaretShow();
@@ -679,7 +711,7 @@ public class TextBox : TextBoxBase
 			ResetView();
 		}
 		// Draw scrollbar corner if both are visible
-		if (vScrollBar != null && hScrollBar != null && vScrollBar.Visible && hScrollBar.Visible)
+		if (vScrollBar != null && hScrollBar != null && showVScrollBar && showHScrollBar)
 		{
 			g.FillRectangle(SystemBrushes.Control, hScrollBar.Right, vScrollBar.Bottom, vScrollBar.Width, hScrollBar.Height);
 		}
@@ -691,16 +723,30 @@ public class TextBox : TextBoxBase
 		}
 
 		bool focused = Focused;
-		if (!Enabled || ReadOnly)
+		
+		if(!Enabled || ReadOnly)
 		{
-			using (Brush disabledBackBrush = new SolidBrush(SystemColors.Control))
+			// if an other color is used for BackColor than SystemColors.Window
+			// use the BackColor to fill, even the control is disabled.
+			Color col;
+			if(BackColor == SystemColors.Window)
+			{
+				col = SystemColors.Control;
+			}
+			else
+			{
+				col = BackColor;
+			}
+			
+			//using (Brush disabledBackBrush = new SolidBrush(SystemColors.Control))
+			using(Brush disabledBackBrush = new SolidBrush(col))
 			{
 				g.FillRegion(disabledBackBrush, g.Clip);
 			}
 		}
 		else
 		{
-			using (Brush backBrush = new SolidBrush(BackColor))
+			using(Brush backBrush = new SolidBrush(BackColor))
 			{
 				g.FillRegion(backBrush, g.Clip);
 			}
@@ -732,6 +778,7 @@ public class TextBox : TextBoxBase
 	}
 
 #if CONFIG_COMPONENT_MODEL	
+	[TODO]
 	private void HandleDataBindingCollectionChanged(object sender, CollectionChangeEventArgs E)
 	{
 		Binding binding = (Binding)E.Element;
@@ -743,10 +790,10 @@ public class TextBox : TextBoxBase
 				binding.PushData();
 				break;
 			case CollectionChangeAction.Refresh:
-				/* TODO: What do we do here? */
+				// Fix: What do we do here?
 				break;
 			case CollectionChangeAction.Remove:
-				/* TODO: Not sure here either? */
+				// Fix: Not sure here either?
 				break;
 		}	
 	}
@@ -764,6 +811,8 @@ public class TextBox : TextBoxBase
 	// All rendered in client coordinates.
 	protected void LayoutFromText(String newText, Graphics g)
 	{
+		bool callSetScrollBarPositions;
+		
 		if (!IsHandleCreated)
 			return;
 		if (layout == null)
@@ -777,6 +826,7 @@ public class TextBox : TextBoxBase
 		// yLine is the y coordinate of this point.
 		int yLine = 1;
 		int posLine = 0;
+
 		for (int i = 0; i < layout.Items.Length; i++)
 		{
 			if (i > newText.Length - 1)
@@ -790,6 +840,13 @@ public class TextBox : TextBoxBase
 			if (newText[i] != Text[i])
 				break;
 		}
+
+		// Set up the scrollbars and remember whether we need to call SetScrollBarPositions.
+		// We need to do this here so TextDrawArea returns correct values.
+		callSetScrollBarPositions = SetupScrollBars();
+
+		graphicsOffset = g.MeasureCharacters("A", Font, new Rectangle(0,0, 100, 100), new StringFormat())[0].X;
+
 		// We leave 1 pixel on the left and right for the caret
 		// Multiline textboxes are infinite in the y direction
 		// non multiline are infinite in the x direction and we scroll when needed
@@ -801,7 +858,7 @@ public class TextBox : TextBoxBase
 		}
 		else
 		{
-			measureBounds = new Rectangle(1, yLine - 1, maxXY, TextDrawArea.Height - (yLine - 1));
+			measureBounds = new Rectangle(1 - graphicsOffset, yLine - 1, maxXY + graphicsOffset - 1, TextDrawArea.Height - (yLine - 1));
 		}
 
 		string measureText = newText.Substring(posLine);
@@ -905,7 +962,10 @@ public class TextBox : TextBoxBase
 			layout.Items[i] = item;
 			prevType = item.type;
 		}
-		SetupScrollBars();
+		
+		// Set the positions of the scrollbars if SetupScrollBars told us to do so
+		if (callSetScrollBarPositions)
+		    SetScrollBarPositions ();
 	}
 
 	// Make sure the caret is visible
@@ -928,7 +988,7 @@ public class TextBox : TextBoxBase
 		}
 		if (caretBounds.Left- XViewOffset < TextDrawArea.Left)
 		{
-			XViewOffset = caretBounds.Left;
+			XViewOffset = caretBounds.Left - 1;
 		}
 		else if (caretBounds.Right - XViewOffset > TextDrawArea.Right)
 		{
@@ -951,7 +1011,7 @@ public class TextBox : TextBoxBase
 			}
 			else if (textAlign == HorizontalAlignment.Right)
 			{
-				XViewOffset = maxXY - TextDrawArea.Width + 2;
+				XViewOffset = maxXY - TextDrawArea.Width - graphicsOffset + 1;
 			}
 		}
 		else
@@ -1120,9 +1180,10 @@ public class TextBox : TextBoxBase
 		CaretSetPosition(GetSelectionStart() + GetSelectionLength());
 	}
 
+	[TODO]
 	internal override void OnToggleInsertMode()
 	{
-		// TODO: change caret appearance
+		// Fix: change caret appearance
 	}
 	
 	// Caret navigation
@@ -1317,16 +1378,26 @@ public class TextBox : TextBoxBase
 		{
 			height = ClientToBounds(Size.Empty).Height + Font.Height + 1;
 		}
+		
+		Rectangle oldRec = Bounds;
+		
+		if( oldRec.X == x && oldRec.Y == y && oldRec.Width == width && oldRec.Height == height ) { // no update needed if same bounds
+			return;
+		}
+		
 		base.SetBoundsCore (x, y, width, height, specified);
 		if (!IsHandleCreated)
 		{
 			return;
 		}
+		
 		// If the height or width changes then relayout the text
 		if ((specified & BoundsSpecified.Height) != 0 | (specified & BoundsSpecified.Width) != 0)
 		{
+			layout = null;
 			LayoutFromText(Text);
 			SetScrollBarPositions();
+			ResetView ();
 			// Redraw
 			InvalidateAll();
 		}
@@ -1373,7 +1444,7 @@ public class TextBox : TextBoxBase
 					// Setup the first values.
 					if (y == int.MinValue)
 					{
-						x = bounds.X;
+						x = bounds.X - graphicsOffset;
 						y = bounds.Y;
 						prevFore = fore;
 					}
@@ -1404,6 +1475,7 @@ public class TextBox : TextBoxBase
 							{
 								lineText = text.Substring(lineStart, cleanI - lineStart + 1);
 							}
+							
 							if (Enabled)
 							{
 								g.DrawString(lineText, font, prevFore, new Point (x, y));
@@ -1414,7 +1486,7 @@ public class TextBox : TextBoxBase
 							}
 						}
 						lineStart = i;
-						x = bounds.X;
+						x = bounds.X - graphicsOffset;
 						y = bounds.Y;
 						prevFore = fore;
 					}
@@ -1581,6 +1653,9 @@ public class TextBox : TextBoxBase
 	// Set update region
 	internal override void CaretSetPosition( int position)
 	{
+		// remember Caret position
+		caretPosition = position;
+
 		if (!IsHandleCreated)
 		{
 			return;
@@ -1942,16 +2017,17 @@ public class TextBox : TextBoxBase
 		layout = null;
 	}
 
+
 	private Rectangle TextDrawArea
 	{
 		get
 		{
 			Rectangle rect = ClientRectangle;
-			if (vScrollBar != null && vScrollBar.Visible)
+			if (vScrollBar != null && showVScrollBar)
 			{
 				rect.Width -= vScrollBar.Width;
 			}
-			if (hScrollBar != null && hScrollBar.Visible)
+			if (hScrollBar != null && showHScrollBar)
 			{
 				rect.Height -= hScrollBar.Height;
 			}
@@ -1996,7 +2072,7 @@ public class TextBox : TextBoxBase
 			// Make sure the entire textbox is redrawn
 			InvalidateAll();
 			yViewOffset = value;
-			if (vScrollBar != null && vScrollBar.Visible)
+			if (vScrollBar != null && showVScrollBar)
 			{
 				vScrollBar.Value = yViewOffset;
 			}
