@@ -1044,8 +1044,7 @@ public class DefaultThemePainter : IThemePainter
 										int steps, int step,
 										int value, bool enabled)
 			{
-				// TODO : draw disabled mode progressbar
-				// TODO : handle large no of blocks ie merge cases
+				// TODO : handle large number of blocks ie merge cases
 				int blockWidth, blockHeight, xSpacing, ySpacing;
 				DrawBorder3D(graphics,x,y,width,height, 
 							 SystemColors.InactiveBorder,
@@ -1538,13 +1537,28 @@ public class DefaultThemePainter : IThemePainter
 			}
 
 	// Draw an image in its disabled state.
-	[TODO]
 	public virtual void DrawImageDisabled
 				(Graphics graphics, Image image,
 				 int x, int y, Color background)
 			{
-				// TODO: create the greyed version of the image.
-				graphics.DrawImage(image, x, y);
+				Imaging.ColorMatrix colorMatrix = new Imaging.ColorMatrix(new float[][] { new float[]{ 0.5f,0.5f,0.5f,0,0 },
+											new float[]{ 0.5f,0.5f,0.5f,0,0 },
+											new float[]{ 0.5f,0.5f,0.5f,0,0 },
+											new float[]{ 0,0,0,1,0,0 },
+											new float[]{ 0,0,0,0,1,0 },
+											new float[]{ 0,0,0,0,0,1 }
+											}
+									);
+
+				Imaging.ImageAttributes imageAttributes = new Imaging.ImageAttributes();
+				imageAttributes.SetColorMatrix(colorMatrix);
+				graphics.DrawImage(image,
+						new Rectangle(x, y, image.Width, image.Height),
+						0, 0,
+						image.Width,
+						image.Height,
+						GraphicsUnit.Pixel,
+						imageAttributes);
 			}
 
 	// Draw a locked selection frame.
@@ -1769,6 +1783,9 @@ public class DefaultThemePainter : IThemePainter
 				 Rectangle decrement, bool decDown,
 				 Rectangle increment, bool incDown)
 			{
+				
+				               
+	
 				// fill in the background
 				//graphics.FillRectangle(backgroundBrush, bounds);
 				if (hatchBrush != null && (foreColor != hatchForeColor ||
@@ -1854,7 +1871,7 @@ public class DefaultThemePainter : IThemePainter
 				ButtonState barState = enabled ?
 				                       ButtonState.Normal :
 				                       ButtonState.Inactive;
-
+				
 				// draw the scroll bar
 				if (track.Height > 0 && bar.IntersectsWith(drawBounds))
 					DrawButton(graphics,
@@ -1862,7 +1879,7 @@ public class DefaultThemePainter : IThemePainter
 				           bar.Width, bar.Height,
 				           barState,
 				           foreColor, backColor,
-				           false);
+				           false);   
 			}
 
 	// Draw a scroll button control.
@@ -1871,13 +1888,15 @@ public class DefaultThemePainter : IThemePainter
 				 ScrollButton button, ButtonState state,
 				 Color foreColor, Color backColor)
 			{
-				// draw the button border
+				
+			        // draw the button border
+				
 				DrawButton(graphics,
 				           x, y, width, height,
 				           state,
 				           foreColor, backColor,
 				           false);
-
+#if false
 				// Get the drawing color and adjust for the pressed state.
 				Color color;
 				Color inactiveColor = ControlPaint.LightLight(backColor);
@@ -1897,28 +1916,36 @@ public class DefaultThemePainter : IThemePainter
 				}
 
 				// Draw the inactive drop-down glyph for the button.
+				
+				if(width==0)width=-1;
+				
 				if((state & ButtonState.Inactive) != 0)
 				{
+				
 					switch (button)
 					{
 						case ScrollButton.Up:
 						{
-							DrawGlyph(graphics, x + 1, y + 1, width, height,
+						
+								DrawGlyph(graphics, x + 1, y + 1, width, height,
 									  Glyphs.up_arrow_bits,
 									  Glyphs.up_arrow_width,
 									  Glyphs.up_arrow_height, inactiveColor);
+
 						}
 						break;
 	
 						case ScrollButton.Down:
 						{
-							DrawGlyph(graphics, x + 1, y + 1, width, height,
+						
+								DrawGlyph(graphics, x + 1, y + 1, width, height,
 									  Glyphs.down_arrow_bits,
 									  Glyphs.down_arrow_width,
 									  Glyphs.down_arrow_height, inactiveColor);
-						}
+						
+							}
 						break;
-	
+	                                        
 						case ScrollButton.Left:
 						{
 							DrawGlyph(graphics, x + 1, y + 1, width, height,
@@ -1943,33 +1970,41 @@ public class DefaultThemePainter : IThemePainter
 				switch (button)
 				{
 					case ScrollButton.Up:
-					{
-						DrawGlyph(graphics, x, y, width, height,
+					{      
+						
+							DrawGlyph(graphics, x, y, width, height,
 								  Glyphs.up_arrow_bits,
 								  Glyphs.up_arrow_width,
 								  Glyphs.up_arrow_height, color);
+						
 					}
 					break;
-
+					 
 					case ScrollButton.Down:
-					{
-						DrawGlyph(graphics, x, y, width, height,
+					{       
+						
+							DrawGlyph(graphics, x, y, width, height,
 								  Glyphs.down_arrow_bits,
 								  Glyphs.down_arrow_width,
 								  Glyphs.down_arrow_height, color);
+
 					}
 					break;
 
 					case ScrollButton.Left:
 					{
+					
+					     
+					   							
 						DrawGlyph(graphics, x, y, width, height,
 								  Glyphs.left_arrow_bits,
 								  Glyphs.left_arrow_width,
-								  Glyphs.left_arrow_height, color);
+					     			  Glyphs.left_arrow_height, color);
+					        							
 					}
 					break;
 
-					case ScrollButton.Right:
+					case ScrollButton.Right:                                                
 					{
 						DrawGlyph(graphics, x, y, width, height,
 								  Glyphs.right_arrow_bits,
@@ -1978,8 +2013,10 @@ public class DefaultThemePainter : IThemePainter
 					}
 					break;
 				}
+#endif
 
-#if false	// May need later for non-standard glyph sizes.
+				// Set non-standard glyph sizes.
+				// The glyphs are resizable using the space left for the increment and decrement buttons.
 				x += 2; // skip border
 				y += 2; // skip border
 				width -= 4; // skip border
@@ -1999,69 +2036,172 @@ public class DefaultThemePainter : IThemePainter
 					y -= 1;
 				}
 
-				// setup the glyph shape
-				int glyphWidth = 3;
-				int glyphHeight = 3;
-				if (button == ScrollButton.Up || button == ScrollButton.Down)
-				{
-					glyphWidth *= 2;
-				}
-				else
-				{
-					glyphHeight *= 2;
-				}
-				Point[] glyph = new Point[3];
-				int offsetX = x+((width-glyphWidth)/2);
-				int offsetY = y+((height-glyphHeight)/2);
+				int glyphWidth = 3; // Glyph width size
+				int glyphHeight = 3; // Glyph height size
 				switch (button)
 				{
 					case ScrollButton.Up:
-					{
-						glyph[0] = new Point(offsetX+3,
-						                     offsetY);
-						glyph[1] = new Point(offsetX+6,
-						                     offsetY+3);
-						glyph[2] = new Point(offsetX,
-						                     offsetY+3);
+				{
+						if (height<1)  // Same but for height (UP and DOWN buttons) 
+						{
+							glyphWidth = 0;
+							glyphHeight = 0;				    
+				}
+						else if (height<=4) // if the glyph is small we draw a single point
+						{
+							glyphWidth = 0;
+							glyphHeight = 1;
+						}
+						else if (height<=12) // We resize the glyph picture using its logical size.
+						{
+							glyphWidth = (int)(3.0*((double)height/12.0));
+							glyphHeight = (int)(3.0*((double)height/12.0));
+						}
 					}
-					break;
+					break;	
+
 
 					case ScrollButton.Down:
 					{
-						glyph[0] = new Point(offsetX,
-						                     offsetY);
-						glyph[1] = new Point(offsetX+6,
-						                     offsetY);
-						glyph[2] = new Point(offsetX+3,
-						                     offsetY+3);
+						if (height<1)  // Same but for height (UP and DOWN buttons) 
+						{
+							glyphWidth = 0;
+							glyphHeight = 0;				    
+						}
+						else if (height<=4) // if the glyph is small we draw a single point
+						{
+							glyphWidth = 0;
+							glyphHeight = 1;
+						}
+						else if (height<=12) // We resize the glysh picture using its logical size.
+				{
+							glyphWidth = (int)(3.0*((double)height/12.0));
+							glyphHeight = (int)(3.0*((double)height/12.0));
+				}
+					}
+					break;	
+					
+					case ScrollButton.Right:
+					{
+						if (width<1) // If the glyph width is too small we don't draw it (LEFT and RIGHT buttons)
+						{
+							glyphWidth = 0;
+							glyphHeight = 0;
+						}
+						else if (width<=4) // if the glyph is small we draw a single point
+						{
+							glyphWidth = 1;
+							glyphHeight = 0;
+						}
+						else if (width<=12) // We resize the glysh picture using its logical size.
+						{
+							glyphWidth = (int)(3.0*((double)width/12.0));
+							glyphHeight = (int)(3.0*((double)width/12.0));
+						}
 					}
 					break;
 
+
 					case ScrollButton.Left:
 					{
-						glyph[0] = new Point(offsetX,
-						                     offsetY+3);
-						glyph[1] = new Point(offsetX+3,
-						                     offsetY);
-						glyph[2] = new Point(offsetX+3,
-						                     offsetY+6);
-					}
-					break;
-
-					case ScrollButton.Right:
-					{
-						glyph[0] = new Point(offsetX,
-						                     offsetY);
-						glyph[1] = new Point(offsetX,
-						                     offsetY+6);
-						glyph[2] = new Point(offsetX+3,
-						                     offsetY+3);
+						if (width<1) // If the glyph width is too small we don't draw it (LEFT and RIGHT buttons)
+						{
+							glyphWidth = 0;
+							glyphHeight = 0;
+						}
+						else if (width<=4) // if the glyph is small we draw a single point
+						{
+							glyphWidth = 1;
+							glyphHeight = 0;
+						}
+						else if (width<=12) // We resize the glyph using its logical size.
+						{
+							glyphWidth = (int)(3.0*((double)width/12.0));
+							glyphHeight = (int)(3.0*((double)width/12.0));
+						}
 					}
 					break;
 
 					default:
 					{
-						throw new ArgumentException("button");
+						throw new ArgumentException("Invalid scroll button");
+					}
+				}
+
+
+				
+
+					
+
+				
+				// setup the glyph shape
+
+				Point[] glyph = new Point[3];
+				int offsetX;
+				int offsetY;
+
+				switch (button)
+				{
+					case ScrollButton.Up:
+					{
+						offsetX = x+width/2-glyphWidth;
+						offsetY = y+(height+1)/2;
+
+						glyph[0] = new Point(offsetX,
+						                     offsetY);
+						glyph[1] = new Point(offsetX+glyphWidth*2,
+						                     offsetY);
+						glyph[2] = new Point(offsetX+glyphWidth,
+						                     offsetY-glyphHeight);
+					}
+					break;
+
+					case ScrollButton.Down:
+					{
+						offsetX = x+width/2-glyphWidth;
+	  					offsetY = y+(height+1)/2-glyphHeight/2;
+
+						glyph[0] = new Point(offsetX,
+						                     offsetY);
+						glyph[1] = new Point(offsetX+glyphWidth*2,
+						                     offsetY);
+						glyph[2] = new Point(offsetX+glyphWidth,
+						                     offsetY+glyphHeight);
+					}
+					break;
+
+					case ScrollButton.Left:
+					{
+
+						offsetX = x+(width+1)/2-glyphWidth/2;
+
+						offsetY = y+((height-glyphHeight*2)/2);
+
+						glyph[0] = new Point(offsetX+glyphWidth,
+						                     offsetY);
+						glyph[1] = new Point(offsetX+glyphWidth,
+						                     offsetY+glyphHeight*2);
+						glyph[2] = new Point(offsetX,
+						                     offsetY+glyphHeight);
+					}
+					break;
+
+					case ScrollButton.Right:
+					{
+						offsetX = x+(width+1)/2;
+						offsetY = y+((height-glyphHeight*2)/2);
+						glyph[0] = new Point(offsetX,
+						                     offsetY);
+						glyph[1] = new Point(offsetX,
+						                     offsetY+glyphHeight*2);
+						glyph[2] = new Point(offsetX+glyphWidth,
+						                     offsetY+glyphHeight);
+					}
+					break;
+
+					default:
+					{
+						throw new ArgumentException("Invalid scroll button");
 					}
 				}
 				
@@ -2079,7 +2219,8 @@ public class DefaultThemePainter : IThemePainter
 				{
 					graphics.DrawPolygon(pen,glyph);
 				}
-#endif
+
+
 			}
 
 	// Draw a selection frame.
@@ -2092,12 +2233,58 @@ public class DefaultThemePainter : IThemePainter
 			}
 
 	// Draw a size grip.
-	[TODO]
 	public virtual void DrawSizeGrip
-				(Graphics graphics, Color backColor,
-				 int x, int y, int width, int height)
+				(Graphics graphics, Color backColor, Rectangle drawBounds)
 			{
-				// TODO
+				Pen light = new Pen(Color.White, 1);
+				Pen dark = new Pen(Color.Black, 1);
+
+				// Backfill accordingly
+				graphics.FillRectangle(new SolidBrush(backColor),
+							drawBounds.X, drawBounds.Y,
+							drawBounds.Width, drawBounds.Height);
+
+				// Top most line
+				graphics.DrawLine(SystemPens.ControlDark,
+						drawBounds.Left + 1,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1 ,
+						drawBounds.Top + 1
+					);
+				graphics.DrawLine(SystemPens.ControlLightLight,
+						drawBounds.Left,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1,
+						drawBounds.Top
+					);
+
+				// Middle line
+				graphics.DrawLine(SystemPens.ControlDark,
+						drawBounds.Left + 5,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1,
+						drawBounds.Top + 5
+					);
+				graphics.DrawLine(SystemPens.ControlLightLight,
+						drawBounds.Left + 4,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1,
+						drawBounds.Top + 4
+					);
+
+				// Bottom line
+				graphics.DrawLine(SystemPens.ControlDark,
+						drawBounds.Left + 10,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1,
+						drawBounds.Top + 10
+					);
+				graphics.DrawLine(SystemPens.ControlLightLight,
+						drawBounds.Left + 9,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1,
+						drawBounds.Top  + 9
+					);
 			}
 
 	// Draw a list box
@@ -2128,7 +2315,7 @@ public class DefaultThemePainter : IThemePainter
 				SolidBrush brush = new SolidBrush(ControlPaint.Light(color));
 				layoutRectangle.Offset(1.0f, 1.0f);
 				graphics.DrawString(s, font, brush, layoutRectangle, format);
-				brush.Color = ControlPaint.Dark(color);
+				brush.Color = ControlPaint.Dark(ControlPaint.Dark(ControlPaint.Dark(color)));
 				layoutRectangle.Offset(-1.0f, -1.0f);
 				graphics.DrawString(s, font, brush, layoutRectangle, format);
 				brush.Dispose();
