@@ -65,7 +65,8 @@ namespace System.Windows.Forms
 		internal int xOffset = 0;
 		const int xPadding = 2;
 		int xScrollValueBeforeEdit = 0;
- 
+		ClickEvent clickEvent;
+
 		public event TreeViewEventHandler AfterCheck
 		{
 			add
@@ -483,11 +484,38 @@ namespace System.Windows.Forms
 							// Draw the node text.
 							if (nodes.currentNode  == selectedNode && (Focused || !hideSelection))
 							{
-								// TODO: FullRowSelect
-								g.FillRectangle(SystemBrushes.Highlight, bounds);
+								Rectangle r;
+
+								// **TODO**
+								// Running into a chicken and egg issue here where if
+								// we fill the rect here, we're overdrawing the
+								// checkBoxes or ExpanderMarker of above
+
+								// Draw FullRowSelect if we qualify
+								if(fullRowSelect && !showLines)
+								{
+									int left = 1;
+//									if(nodes.currentNode)
+//									{
+//										left = 10;
+//									}
+									g.FillRectangle(SystemBrushes.Highlight, new Rectangle(left, bounds.Y, drawableWidth - 1, bounds.Height));
+								}
+								else
+								{
+									g.FillRectangle(SystemBrushes.Highlight, bounds);
+								}
 								g.DrawString(nodes.currentNode.Text, Font, SystemBrushes.HighlightText, bounds, format);
+
 								// Draw the focus rectangle.
-								Rectangle r = new Rectangle(bounds.X - 1, bounds.Y - 1, bounds.Width + 1, bounds.Height + 1);
+								if(FullRowSelect && !ShowLines)
+								{
+									r = new Rectangle(0, bounds.Y - 1, drawableWidth, bounds.Height + 1);
+								}
+								else
+								{
+									r = new Rectangle(bounds.X - 1, bounds.Y - 1, bounds.Width + 1, bounds.Height + 1);
+								}
 								ControlPaint.DrawFocusRectangle(g, r);
 							}
 							else
@@ -1069,7 +1097,7 @@ namespace System.Windows.Forms
 			}
 		}
 		
-		protected virtual void OnAfterExpand(TreeViewEventArgs e)
+		protected internal virtual void OnAfterExpand(TreeViewEventArgs e)
 		{
 			TreeViewEventHandler handler = (TreeViewEventHandler)GetHandler(EventId.AfterExpand);
 			if (handler != null)
@@ -1114,7 +1142,7 @@ namespace System.Windows.Forms
 			}
 		}
 
-		protected virtual void OnBeforeExpand(TreeViewCancelEventArgs e)
+		protected internal virtual void OnBeforeExpand(TreeViewCancelEventArgs e)
 		{
 			TreeViewCancelEventHandler handler = (TreeViewCancelEventHandler)GetHandler(EventId.BeforeExpand);
 			if (handler != null)
@@ -1249,7 +1277,7 @@ namespace System.Windows.Forms
 			}
 			else
 			{
-				ProcessClick(e.X, e.Y, true);
+				ProcessClick(e.X, e.Y, true, ClickEvent.None);
 			}
 			base.OnMouseDown (e);
 		}
@@ -1271,12 +1299,12 @@ namespace System.Windows.Forms
 		// Non Microsoft member.
 		protected override void OnMouseUp(MouseEventArgs e)
 		{
-			ProcessClick(e.X, e.Y, (e.Button == MouseButtons.Right));
+			ProcessClick(e.X, e.Y, (e.Button == MouseButtons.Right), clickEvent);
 
 			base.OnMouseUp (e);
 		}
 
-		void ProcessClick(int x, int y, bool rightMouse)
+		void ProcessClick(int x, int y, bool rightMouse, ClickEvent clickEvent)
 		{
 			int nodeFromTop = -1;
 			int height = ItemHeight;
@@ -1295,9 +1323,18 @@ namespace System.Windows.Forms
 					{
 						// Check if the y matches this node.
 						if (y < height * (nodeFromTop + 1))
-						{
+						{							
 							bool allowEdit = false;
 							bool allowSelect = true;
+							// Raise click or double click when called from MouseUp
+							if(clickEvent == ClickEvent.Click)
+							{
+								OnClick(EventArgs.Empty);
+							}
+							else if(clickEvent == ClickEvent.DoubleClick)
+							{
+								OnDoubleClick(EventArgs.Empty);
+							}
 							// Clicking the image can be used to select.
 							if (imageList == null || !GetImageBounds(nodeFromTop, nodes.level).Contains(x, y))
 							{
@@ -1826,6 +1863,8 @@ namespace System.Windows.Forms
 			{
 				OnMouseDown(new MouseEventArgs(buttons, clicks, x, y, delta));
 			}
+			// Set event to be raised on mouse up
+			clickEvent = (clicks == 1) ? ClickEvent.Click : ClickEvent.DoubleClick;
 		}
 
 		public TreeNode TopNode 
@@ -1982,6 +2021,13 @@ namespace System.Windows.Forms
 				}
 			}
 		}
+		
+		internal enum ClickEvent
+		{
+			Click,
+			DoubleClick,
+			None
+		}
 	}
 
 }
