@@ -599,13 +599,15 @@ class XmlDocument : XmlNode
 				return new XmlDeclaration
 					(placeholder, version, encoding, standalone);
 			}
-	
+
+#if CONFIG_XPATH
 	// NOTE: should be protected internal virtual 
 	// but for ml-pnet's System.Data the internal modifier has to be removed
 	protected virtual XPathNavigator CreateNavigator(XmlNode node)
 			{
 				return node.CreateNavigator();
 			}
+#endif
 
 	// Get an element by identifier.
 	[TODO]
@@ -683,7 +685,8 @@ class XmlDocument : XmlNode
 				{
 					throw new XmlException(S._("XmlException_NoXml"));
 				}
-
+				
+				RemoveAll();
 				XmlTextReader reader = new XmlTextReader(xml, XmlNodeType.Document, null);
 				BuildStructure(reader);
 			}
@@ -929,7 +932,7 @@ class XmlDocument : XmlNode
 							{
 								if(r.NodeType != XmlNodeType.Whitespace)
 								{
-									return ReadNodeInternal(r);
+									return null;
 								}
 							}
 							return null;
