@@ -28,8 +28,38 @@ public sealed class ApplicationIdentity
 {
 	// TODO
 
+	// for use by Microsoft/Internal/Deployment/InternalApplicationIdentityHelper.cs
 	internal Object id;
 
+	private String fullName;
+
+	public ApplicationIdentity(String applicationIdentityFullName)
+	{
+		// this is the excception thrown in the beta 1 of the SDK
+		if(applicationIdentityFullName == null)
+		{
+			throw new NullReferenceException();
+		}
+		this.fullName = applicationIdentityFullName;
+	}
+
+	[TODO]
+	public String CodeBase
+	{
+		get
+		{
+			return null;
+		}
+	}
+
+	[TODO]
+	public String FullName
+	{
+		get
+		{
+			return fullName;
+		}
+	}
 }; // class ApplicationIdentity
 
 #endif // CONFIG_FRAMEWORK_2_0
